/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.assistant;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.aktien.Aktien;
import de.frankmuenster.mahoe.banks.aktien.assistant.AktienCreateAccountAssistent;
import de.frankmuenster.mahoe.banks.bos.BankOfScotland;
import de.frankmuenster.mahoe.banks.bos.assistant.BosCreateAccountAssistent;
import de.frankmuenster.mahoe.banks.bsh.SchwaebischHall;
import de.frankmuenster.mahoe.banks.bsh.assistant.BshCreateAccountAssistent;
import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.banks.santander.assistant.SantanderCreateAccountAssistent;
import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.banks.targobank.assistant.TargoBankCreateAccountAssistent;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Ein Assistent, der bei der Kontoerstellung hilft.
 *
 * @author Frank Mnster
 */
public final class CreateAccountAssistantAction implements Action {

    /**
     * berprft ob schon ein MahoeMain Konto existiert und started ggf. den
     * Kontoanlage-Assistenten.
     *
     * @param blz
     */
    private static void createAccount(final String blz) {

        CreateAccountAssistant createAssistent = null;
        switch (blz) {
        case Aktien.BLZ:
            createAssistent = new AktienCreateAccountAssistent();
            break;
        case BankOfScotland.BLZ:
            createAssistent = new BosCreateAccountAssistent();
            break;
        case SantanderBank.BLZ:
            createAssistent = new SantanderCreateAccountAssistent();
            break;
        case SchwaebischHall.BLZ:
            createAssistent = new BshCreateAccountAssistent();
            break;
        case TargoBank.BLZ:
            createAssistent = new TargoBankCreateAccountAssistent();
            break;
        default:
            break;
        }
        if (createAssistent != null) {
            final String bankName = MahoeSynchronizeBackend.getBlzSupported().get(blz);
            try {
                final Konto konto = createAssistent.createAccount();
                GUI.startView(de.willuhn.jameica.hbci.gui.views.KontoNew.class, konto);
                final StatusBarMessage statusMsg = new StatusBarMessage(
                        I18N.tr("CreateAccountAssisstantAction.createAccountSuccessful", bankName), //$NON-NLS-1$
                        StatusBarMessage.TYPE_SUCCESS);
                Application.getMessagingFactory().sendMessage(statusMsg);
            } catch (final CreateAccountException e) {
                Logger.error(I18N.tr("CreateAccountAssisstantAction.createAccountFailed", e.getMessage()), e); //$NON-NLS-1$
                // Benutzer ber Fehler beim Anlegen des Kontos benachrichtigen
                final StatusBarMessage statusMsg = new StatusBarMessage(
                        I18N.tr("CreateAccountAssisstantAction.createAccountFailedStatusBarMsg", bankName), //$NON-NLS-1$
                        StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(statusMsg);
                try {
                    final String msg = I18N.tr("CreateAccountAssisstantAction.createAccountFailedPopupMsg"); //$NON-NLS-1$
                    Application.getCallback().notifyUser(msg);
                } catch (final Exception e1) { // NOPMD
                    // Nur loggen
                    Logger.error(I18N.tr("CreateAccountAssisstantAction.userInfoFailed", e.getMessage()), e); //$NON-NLS-1$
                }
            }
        }
    }

    /**
     * Standardkonstruktor
     */
    public CreateAccountAssistantAction() {

        // absichtlich leer
    }

    /**
     * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
     */
    @Override
    public void handleAction(final Object context) throws ApplicationException {

        // Erst mal die Bank heraus finden
        final CreateKontoStartDialog startDialog = new CreateKontoStartDialog();
        try {
            final String blz = startDialog.open();
            CreateAccountAssistantAction.createAccount(blz);
        } catch (final OperationCanceledException e) {
            Logger.info(I18N.tr("CreateAccountAssisstantAction.cancelledByUser")); //$NON-NLS-1$
        } catch (final Exception e) { // NOPMD
            Logger.error(I18N.tr("CreateAccountAssisstantAction.openDialogFailed", e.getMessage()), e); //$NON-NLS-1$
        }
    }
}
