/*******************************************************************************
 * Copyright 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.gui.parts;

import org.apache.commons.lang.StringUtils;

/**
 * @author Frank Mnster
 */
public class AktienListEntry {

    /**
     * The amount of stocks
     */
    private String amount;

    /**
     * Der Brsenplatz
     */
    private String exchange;

    /**
     * Der Name des Wertpapiers
     */
    private String name;

    /**
     * Die ID des Wertpapiers
     */
    private String stockId;

    /**
     * ISIN of the stock
     */
    private String isin;

    /**
     * WKN of the stock
     */
    private String wkn;

    /**
     * Main URL of the stock
     */
    private String mainUrl;

    /**
     * Standardkonstruktor
     */
    public AktienListEntry() {
        // Absichtlich leer
    }

    /**
     * Standardkonstruktor
     *
     * @param name
     * @param exchange
     * @param amount
     * @param stockId
     */
    public AktienListEntry(final String name, final String exchange, final String amount, final String stockId,
            final String isin, final String wkn, final String mainUrl) {
        this.name = name;
        this.exchange = exchange;
        this.amount = amount;
        this.stockId = stockId;
        this.isin = isin;
        this.wkn = wkn;
        this.mainUrl = mainUrl;
    }

    /**
     * @return the amount
     */
    public String getAmount() {

        return this.amount;
    }

    /**
     * @return the exchange
     */
    public String getExchange() {

        return this.exchange;
    }

    /**
     * Gets the exchange shortened to 10 characters if needed.
     *
     * @return the shortened exchange
     */
    public String getExchangeShort() {

        return StringUtils.abbreviate(this.exchange, 10);
    }

    /**
     * @return the isin
     */
    public String getIsin() {
        return this.isin;
    }

    /**
     * @return the mainUrl
     */
    public String getMainUrl() {
        return this.mainUrl;
    }

    /**
     * @return the name
     */
    public String getName() {

        return this.name;
    }

    /**
     * Gets the name shortened to 20 charters if needed, but with a length right
     * padded to 21.
     *
     * @return the shortened and right padded name
     */
    public String getNameShort() {

        String nameShort = StringUtils.abbreviate(this.name, 20);
        nameShort = StringUtils.rightPad(nameShort, 21);
        return nameShort;
    }

    /**
     * @return the stockId
     */
    public String getStockId() {

        return this.stockId;
    }

    /**
     * @return the wkn
     */
    public String getWkn() {
        return this.wkn;
    }

    /**
     * @param amount
     *            the amount to set
     */
    public void setAmount(final String amount) {

        this.amount = amount;
    }

    /**
     * @param exchange
     *            the exchange to set
     */
    public void setExchange(final String exchange) {

        this.exchange = exchange;
    }

    /**
     * @param isin
     *            the isin to set
     */
    public void setIsin(final String isin) {
        this.isin = isin;
    }

    /**
     * @param mainUrl
     *            the mainUrl to set
     */
    public void setMainUrl(final String mainUrl) {
        this.mainUrl = mainUrl;
    }

    /**
     * @param name
     *            the name to set
     */
    public void setName(final String name) {

        this.name = name;
    }

    /**
     * @param stockId
     *            the stockId to set
     */
    public void setStockId(final String stockId) {

        this.stockId = stockId;
    }

    /**
     * @param wkn
     *            the wkn to set
     */
    public void setWkn(final String wkn) {
        this.wkn = wkn;
    }
}
