/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.gui.parts;

import java.lang.reflect.Field;
import java.rmi.RemoteException;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.TableItem;

import de.frankmuenster.mahoe.banks.aktien.gui.menu.AktienSettingsAktienTabelleContextMenu;
import de.frankmuenster.mahoe.banks.aktien.gui.menu.AktienTabelleContextMenuListener;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class AktienSettingsAktienTabellePart extends TablePart {

    /**
     * Standardkonstruktor
     *
     * @param list
     * @param action
     * @param listener
     */
    public AktienSettingsAktienTabellePart(final List<AktienListEntry> list, final Action action,
            final AktienTabelleContextMenuListener listener) {
        super(list, action);

        // Set basic behaviour
        this.setRememberColWidths(true);
        this.setRememberOrder(false);
        this.setRememberState(false);
        // Add columns
        this.addColumn(I18N.tr("AktienSettingsAktienTabellePart.name"), "nameShort"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("AktienSettingsAktienTabellePart.exchange"), "exchangeShort"); //$NON-NLS-1$ //$NON-NLS-2$
        this.addColumn(I18N.tr("AktienSettingsAktienTabellePart.amount"), "amount"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("AktienSettingsAktienTabellePart.stockId"), "stockId"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("AktienSettingsAktienTabellePart.isin"), "isin"); //$NON-NLS-1$//$NON-NLS-2$
        this.addColumn(I18N.tr("AktienSettingsAktienTabellePart.wkn"), "wkn"); //$NON-NLS-1$//$NON-NLS-2$
        // add context menu
        final AktienSettingsAktienTabelleContextMenu contextMenu = new AktienSettingsAktienTabelleContextMenu();
        contextMenu.addListener(listener);
        this.setContextMenu(contextMenu);
    }

    /**
     * @see de.willuhn.jameica.gui.parts.TablePart#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public synchronized void paint(final Composite parent) throws RemoteException { // NOPMD

        super.paint(parent);
        final Composite composit = parent;
        // get table from superclass
        Field tableField;
        try {
            tableField = this.getClass().getSuperclass().getDeclaredField("table"); //$NON-NLS-1$
            tableField.setAccessible(true); // NOPMD - is needed in this case
            final Object table1 = tableField.get(this);
            if (table1 instanceof org.eclipse.swt.widgets.Table) {
                final org.eclipse.swt.widgets.Table tab = (org.eclipse.swt.widgets.Table) table1;
                Logger.debug("Super, hin knnen wir noch einen Listener dranhngen"); //$NON-NLS-1$
                final AbstractTooltipListener tListener = new AbstractTooltipListener(composit) {

                    /**
                     * @see de.frankmuenster.mahoe.banks.aktien.gui.parts.AbstractTooltipListener#computeTextFor(org.eclipse.swt.widgets.Item)
                     */
                    @Override
                    protected String computeTextFor(final Item item) {

                        String text = StringUtils.EMPTY;
                        if (item.getData() instanceof AktienListEntry) {
                            final AktienListEntry entry = (AktienListEntry) item.getData();
                            text = entry.getName();
                        }
                        return text;
                    }

                    /**
                     * @see de.frankmuenster.mahoe.banks.aktien.gui.parts.AbstractTooltipListener#getItemLocation(org.eclipse.swt.widgets.Item)
                     */
                    @Override
                    protected Point getItemLocation(final Item item) {

                        final Rectangle rect = ((TableItem) item).getBounds(0);
                        return tab.toDisplay(rect.x, rect.y);
                    }

                    /**
                     * @see de.frankmuenster.mahoe.banks.aktien.gui.parts.AbstractTooltipListener#retrieveItem(org.eclipse.swt.widgets.Event)
                     */
                    @Override
                    protected Item retrieveItem(final Event event) {

                        return tab.getItem(new Point(event.x, event.y));
                    }

                    /**
                     * @see de.frankmuenster.mahoe.banks.aktien.gui.parts.AbstractTooltipListener#selectItem(org.eclipse.swt.widgets.Item)
                     */
                    @Override
                    protected void selectItem(final Item item) {

                        tab.setSelection(new TableItem[] { (TableItem) item });
                    }
                };
                tab.addListener(SWT.Dispose, tListener);
                tab.addListener(SWT.KeyDown, tListener);
                tab.addListener(SWT.MouseMove, tListener);
                tab.addListener(SWT.MouseHover, tListener);
            }
            Logger.debug(table1.toString());
        } catch (NoSuchFieldException | SecurityException | IllegalArgumentException | IllegalAccessException e) {
            throw new RemoteException("Cannot create Tabelpart", e); //$NON-NLS-1$

        }
    }
}
