/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.gui.parts;

import java.rmi.RemoteException;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.finanzen.boerseard.ArdStockInfoBean;
import de.frankmuenster.jameica.finanzen.boerseard.BoerseArdApi;
import de.frankmuenster.mahoe.gui.utils.SWTFontUtils;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class BoerseArdSelectExchangePart implements Part {

    /**
     * Unsere Listeners fr SearchResultSelected
     */
    private final transient Set<ExchangeSelectListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Liste mit StockInfo's
     */
    protected transient Map<String, String> resultList = new ConcurrentHashMap<>();

    /**
     * Zeilen der Ergebnisliste
     */
    private transient int resultRows;

    /**
     * Die Liste mit den Suchergebnissen
     */
    protected transient org.eclipse.swt.widgets.List exchangeList;

    /**
     * Standardkonstruktor
     *
     * @param listner
     */
    public BoerseArdSelectExchangePart(final ExchangeSelectListener listner) {

        this.addListener(listner);
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            SearchResultSelectListener der hinzugefgt werden soll
     */
    public final void addListener(final ExchangeSelectListener listener) {

        this.listeners.add(listener);
    }

    /**
     * Schreibt die Brsen in die Brsenliste. Alte Eintrge werden gelscht.
     *
     * @param bean
     *            die Brsen
     */
    public void fillList(final ArdStockInfoBean bean) {

        final Map<String, String> exchanges = bean.getExchanges();

        this.exchangeList.removeAll();
        if (exchanges != null && !exchanges.isEmpty()) { // NOPMD on 22.02.21, 19:54
            this.resultList = exchanges;
        } else {
            this.resultList.clear();
            if (BoerseArdApi.NO_STOCK_ID_AVAILABLE.equals(bean.getStockExchangeId())) {
                this.resultList.put("Nicht angegeben", BoerseArdApi.NO_STOCK_ID_AVAILABLE);
            } else {
                this.resultList.put(bean.getExchange(), BoerseArdApi.SINGLE_STOCK_ID);
            }
        }

        for (final String exchange : this.resultList.keySet()) {
            this.exchangeList.add(exchange);
        }
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        if (parent == null) {
            Logger.error(I18N.tr("AbstractStockSearchPart.logParentIsNull")); //$NON-NLS-1$
            return;
        }
        final GridLayout gridLayout = new GridLayout(1, false);
        parent.setLayout(gridLayout);
        final Label nameLabel = new Label(parent, SWT.NONE);
        nameLabel.setText(I18N.tr("AbstractStockSearchPart.lblName")); //$NON-NLS-1$
        final Label searchResultLabel = new Label(parent, SWT.NONE);
        searchResultLabel.setText(I18N.tr("AbstractStockSearchPart.lblSearchResults")); //$NON-NLS-1$
        GridData gridData = new GridData();
        gridData.verticalAlignment = SWT.BEGINNING;
        searchResultLabel.setLayoutData(gridData);
        this.exchangeList = new org.eclipse.swt.widgets.List(parent, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
        gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalAlignment = SWT.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = SWT.FILL;
        gridData.grabExcessVerticalSpace = true;
        this.exchangeList.setLayoutData(gridData);
        Font font = GUI.getDisplay().getSystemFont();
        try {
            font = SWTFontUtils.getMonospacedFont();
        } catch (AssertionError | RuntimeException ex) { // NOPMD
            Logger.warn(I18N.tr("AbstractStockSearchPart.monospacedFontNotFound")); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(I18N.tr("AbstractStockSearchPart.monospacedFontNotFoundMsg"), //$NON-NLS-1$
                            StatusBarMessage.TYPE_INFO));
        }
        final FontData fontData = font.getFontData()[0];
        fontData.setHeight(8);
        font = new Font(parent.getDisplay(), fontData);
        this.exchangeList.setFont(font);
        // Gre festlegen
        for (int i = 0; i < this.resultRows; i++) {
            this.exchangeList.add(StringUtils.EMPTY);
        }
        this.exchangeList.addSelectionListener(new ExchangeSelectionListener());
    }

    /**
     * Clear the list
     */
    public void removeAll() {
        this.exchangeList.removeAll();
    }

    /**
     * Selects the exchange (if present) and notifies listeners
     *
     * @param exchange
     */
    public void selectExchange(final String exchange) {

        String searchExchange = exchange;
        if ("Fondsgesellschaft".equals(searchExchange)) { //$NON-NLS-1$
            searchExchange = "Investmentfonds"; //$NON-NLS-1$
        }
        final int row = this.exchangeList.indexOf(searchExchange);
        if (row >= 0) {
            this.exchangeList.select(row);
            final String exchangeFull = this.exchangeList.getItem(row);
            final Pair<String, String> sel = new ImmutablePair<>(exchangeFull, this.resultList.get(exchangeFull));
            this.notifyListeners(sel);

        }

    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe dem gefundenen
     * QuoteLookupInfoBean Element
     *
     * @param sel
     */
    protected void notifyListeners(final Pair<String, String> sel) {

        for (final ExchangeSelectListener listener : this.listeners) {
            listener.exchangeSelected(sel);
        }
    }

    /**
     * Listener zum Anzeigen, dass ein ein Suchergebnis angeklickt wurde.
     *
     * @author Frank Mnster
     */
    public interface ExchangeSelectListener {

        /**
         * Gibt das angeklickte StockInfo Element zurck
         *
         * @param exchange
         */
        void exchangeSelected(Pair<String, String> exchange);
    }

    /**
     * Auswahllistener der anzeigt, dass ein Suchergebniss angeklickt wurde.
     *
     * @author Frank Mnster
     */
    protected class ExchangeSelectionListener implements SelectionListener {

        /**
         * Standartkonstruktor
         */
        public ExchangeSelectionListener() {

            // absichtlich leer
        }

        /**
         * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org
         *      .eclipse.swt.events.SelectionEvent)
         */
        @Override
        public void widgetDefaultSelected(final SelectionEvent arg0) {

            // nichts zu tun hier
        }

        /**
         * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse
         *      .swt.events.SelectionEvent)
         */
        @Override
        public void widgetSelected(final SelectionEvent arg0) {

            final int index = BoerseArdSelectExchangePart.this.exchangeList.getSelectionIndex();
            final String exchange = BoerseArdSelectExchangePart.this.exchangeList.getItem(index);
            final Pair<String, String> sel = new ImmutablePair<>(exchange,
                    BoerseArdSelectExchangePart.this.resultList.get(exchange));
            BoerseArdSelectExchangePart.this.notifyListeners(sel);
        }
    }
}
