/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.mahoe.banks.bos.assistant;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.mahoe.assistant.AbstractCreateAccountDialog;
import de.frankmuenster.mahoe.assistant.CreateInfo;
import de.frankmuenster.mahoe.banks.bos.BankOfScotland;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.hbci.HBCIProperties;
import de.willuhn.jameica.hbci.gui.input.AccountInput;
import de.willuhn.jameica.hbci.rmi.KontoType;

/**
 * Specifics for the account creation dialog for the Bank of Scotland
 *
 * @author Frank Mnster
 */
public class BosCreateAccountDialog extends AbstractCreateAccountDialog<BosCreateInfo> {

    /**
     * Account number Input
     */
    private transient AccountInput accountNumber;

    /**
     * Login Input
     */
    private transient TextInput login;

    /**
     * Standardkonstruktor
     */
    public BosCreateAccountDialog() {

        super();
        this.setTitle(I18N.tr("BosCreateAccountDialog.bosAsistantDialogTitel")); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractCreateAccountDialog#getData()
     */
    @Override
    protected BosCreateInfo getData() {

        final CreateInfo info = super.getData();
        this.data.setOwner(info.getOwner());
        this.data.setComment(info.getComment());
        this.data.setAccountType(KontoType.GIRO);

        return this.data;

    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractCreateAccountDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) {

        super.paint(parent);

        this.accountNumber = new AccountInput(StringUtils.EMPTY, 10);
        this.accountNumber.setMandatory(true);
        this.accountNumber.setComment(I18N.tr("CreateAccountDialog.accountNumber")); //$NON-NLS-1$

        this.login = new TextInput(StringUtils.EMPTY, 50, I18N.tr("BosCreateAccountDialog.lblLogin")); //$NON-NLS-1$
        this.login.setMandatory(true);
        this.login.setComment(I18N.tr("CreateAccountDialog.loginComment")); //$NON-NLS-1$

        this.container.addLabelPair(I18N.tr("CreateAccountDialog.accountNumber"), this.accountNumber); //$NON-NLS-1$
        this.container.addLabelPair(I18N.tr("BosCreateAccountDialog.lblLogin"), this.login); //$NON-NLS-1$

        this.container.addText(" ", true); //$NON-NLS-1$
        final StringBuilder formString = new StringBuilder(300).append("<form><p>") // NOPMD //$NON-NLS-1$
                .append(I18N.tr("CreateAccountDialog.websiteHint")) //$NON-NLS-1$
                .append("<br/><a href=\"http://progs.frankmuenster.de/mahoe/targo-bank\">http://progs.frankmuenster.de/mahoe/bank-of-scottland</a><br/></p></form>"); //$NON-NLS-1$
        final FormTextPart text = new FormTextPart();
        text.setText(formString.toString());
        this.container.addPart(text);

    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractCreateAccountDialog#prepareData()
     */
    @Override
    protected boolean prepareData() {

        boolean dataOK = true;

        this.data = new BosCreateInfo();
        this.data.setAccountNumber((String) this.accountNumber.getValue());
        this.data.setLogin((String) this.login.getValue());
        if (StringUtils.isEmpty(this.data.getAccountNumber())
                || !HBCIProperties.checkAccountCRC(BankOfScotland.BLZ, this.data.getAccountNumber())) {
            dataOK = false;
            this.accountNumber.setComment(I18N.tr("CreateAccountDialog.accountNumberWrong")); //$NON-NLS-1$
        } else {
            this.accountNumber.setComment(StringUtils.EMPTY);
        }
        if (StringUtils.isEmpty(this.data.getLogin())) {
            dataOK = false;
            this.login.setComment(I18N.tr("CreateAccountDialog.loginWrong")); //$NON-NLS-1$
        } else {
            this.login.setComment(StringUtils.EMPTY);
        }
        return dataOK;

    }

}
