/**
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bos.webpages;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebDriverException;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.bos.web.BosWebAccountInformation;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.AbstractAccountPage;
import de.willuhn.logging.Logger;

/**
 * The account page the account provided for a given account
 *
 * @author papa
 *
 */
public class BosAccountPage extends AbstractAccountPage<BosWebAccountInformation> {

    /**
     * id name for the navigation form
     */
    private static final String ID_NAV_FORM = "navForm"; //$NON-NLS-1$

    /**
     * Creates a new AccountPage for account provided within the given account
     * information
     *
     * @param driver
     *            the Selenium web driver to use
     * @param accountInfo
     *            the account information object
     */
    public BosAccountPage(final WebDriver driver, final BosWebAccountInformation accountInfo) {
        super(driver, accountInfo);

    }

    @Override
    public void open() throws WebSiteException {

        if (!elementExist(By.id(BosAccountPage.ID_NAV_FORM))) {
            Utils.waitIgnoreThreadInterrupts(5000);
            if (!elementExist(By.id(BosAccountPage.ID_NAV_FORM))) {
                Utils.waitIgnoreThreadInterrupts(5000);
                throw new WebDriverException(I18N.tr("BosAccountPage.UnableToOpenOverview")); //$NON-NLS-1$
            }
        }

        final WebElement navigation = this.driver.findElement(By.id(BosAccountPage.ID_NAV_FORM));
        final WebElement accountsLink = navigation.findElement(By.linkText("Tagesgeldkonto")); //$NON-NLS-1$

        accountsLink.click();

        // Open overview as sometimes the page is not loaded correctly
        final WebElement overview = this.driver.findElement(By.linkText("bersicht")); //$NON-NLS-1$

        overview.click();

        final WebElement check = this.driver.findElement(By.id("contentForm")); //$NON-NLS-1$

        Logger.debug(check.getText());
        if (!check.getText().contains("Tagesgeld")) { //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("BosAccountPage.errorOpenOverview"), //$NON-NLS-1$
                    WebSiteExceptionTypes.WEBSITE_NOT_REACHABLE);

        }

    }

    /**
     * Open the details for the given account
     *
     * @return
     */
    public WebElement openDetails() {

        WebElement accountDetails = null;
        final WebElement contentForm = this.driver.findElement(By.id("contentForm")); //$NON-NLS-1$

        final List<WebElement> sections = contentForm.findElements(By.tagName("section")); //$NON-NLS-1$
        for (final WebElement section : sections) {

            // Check IBAN
            final WebElement iban = section.findElement(By.xpath("div/div/div[2]/div/div[2]/div[2]")); //$NON-NLS-1$
            Logger.debug("IBAN: " + iban.getText() + " . " + this.accountInfo.getIban()); //$NON-NLS-1$ //$NON-NLS-2$
            if (iban.getText().replace(StringUtils.SPACE, StringUtils.EMPTY).equals(this.accountInfo.getIban())) {
                accountDetails = section;
                break;
            }
        }

        // TODO: Check if section is open

        return accountDetails;
    }

}
