/*******************************************************************************
 * Copyright 2015 - 2021 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bos.webpages;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebDriverException;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.bos.web.BosWebAccountInformation;
import de.frankmuenster.mahoe.banks.santander.webpages.AbstractPostBoxPage;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.logging.Logger;

/**
 * The post box page for the Bank of Scotland
 *
 * @author Frank Mnster
 * @since 2.6.0
 *
 */
public class BosPostBoxPage extends AbstractPostBoxPage<BosWebAccountInformation> {

    /**
     * ID of navigation form
     */
    private static final String NAV_FORM = "navForm"; //$NON-NLS-1$

    /**
     * Instantiate a new BosPostBoxPage and loads all documents/announcements
     *
     * @param driver
     *            the Selenium WebDriver with the logged in account
     * @param accountInfo
     *            the BosWebAccountInformation object
     */
    public BosPostBoxPage(final WebDriver driver, final BosWebAccountInformation accountInfo) {
        super(driver, accountInfo);

        // open the post box page
        openPostBox();

        // load documents
        loadDocuments();

    }

    private void loadDocuments() {

        // Unread mails
        final WebElement navigation = this.driver.findElement(By.id(BosPostBoxPage.NAV_FORM));
        final WebElement read = navigation.findElement(By.partialLinkText("Neu")); //$NON-NLS-1$
        read.click();

        final WebElement table = this.driver.findElement(By.id("contentForm")); //$NON-NLS-1$
        if (!table.getText().contains("keine ungelesenen")) { //$NON-NLS-1$

            // TODO copy and paste table read from read mails
            Logger.debug(table.getText());
        }

        // read mails

        // TODO Auto-generated method stub

    }

    private void openPostBox() {

        Logger.debug("Open PostBox"); //$NON-NLS-1$

        if (!elementExist(By.id(BosPostBoxPage.NAV_FORM))) {
            Utils.waitIgnoreThreadInterrupts(5000);
            if (!elementExist(By.id(BosPostBoxPage.NAV_FORM))) {
                Utils.waitIgnoreThreadInterrupts(5000);
                throw new WebDriverException("Postbox konnte nicht geffnet werden."); //$NON-NLS-1$
            }
        }

        final WebElement navigation = this.driver.findElement(By.id(BosPostBoxPage.NAV_FORM));
        final WebElement postBoxLink = navigation.findElement(By.partialLinkText("Postbox")); //$NON-NLS-1$
        postBoxLink.click();

        Logger.debug("PostBox open"); //$NON-NLS-1$
    }

}
