
package de.frankmuenster.mahoe.banks.bsh.json.deserializer;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.Locale;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;

/**
 * Deserializer fr die Betrge mit Vorzeichen der BSH
 *
 * @author Frank Mnster/Quirin Huber
 */
public class MoneyDeserializer extends JsonDeserializer<BigDecimal> {

    /**
     * Betragsformat der BSH
     */
    private static final String PATTERN = "+ #,##0.00;- #,##0.00"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public MoneyDeserializer() {
	super();
    }

    /**
     * @see JsonDeserializer#deserialize(JsonParser, DeserializationContext)
     */
    @Override
    public BigDecimal deserialize(final JsonParser jsonParser, final DeserializationContext context)
	    throws IOException {

	final NumberFormat numberFormat = NumberFormat.getNumberInstance(Locale.GERMAN);
	final DecimalFormat format = (DecimalFormat) numberFormat;
	format.applyPattern(MoneyDeserializer.PATTERN);
	format.setParseBigDecimal(true);

	final String textValue = jsonParser.getText();
	return (BigDecimal) format.parse(textValue, new ParsePosition(0));
    }
}
