
package de.frankmuenster.mahoe.banks.bsh.json.deserializer;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.Locale;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;

/**
 * Deserializer fr die Geld-Betrge ohne Vorzeichen der BSH
 *
 * @author Frank Mnster
 */
public class SimpleMoneyDeserializer extends JsonDeserializer<BigDecimal> {

    /**
     * Geldbetragsformat der BSH
     */
    private static final String PATTERN = "#,##0.00"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public SimpleMoneyDeserializer() {
	super();
    }

    /**
     * @see JsonDeserializer#deserialize(JsonParser, DeserializationContext)
     */
    @Override
    public BigDecimal deserialize(final JsonParser jsonParser, final DeserializationContext arg1) throws IOException {

	final NumberFormat numberFormat = NumberFormat.getNumberInstance(Locale.GERMAN);
	final DecimalFormat format = (DecimalFormat) numberFormat;
	format.applyPattern(SimpleMoneyDeserializer.PATTERN);
	format.setParseBigDecimal(true);

	final String textValue = jsonParser.getText();
	return (BigDecimal) format.parse(textValue, new ParsePosition(0));
    }

}
