/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.jobs;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobProvider;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobProviderKontoauszug;
import de.willuhn.annotation.Lifecycle;
import de.willuhn.annotation.Lifecycle.Type;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;

/**
 * Implementierung des Job-Providers fr den Abruf von Kontoauszuegen der
 * Santander.
 *
 * @author Frank Mnster
 */
@Lifecycle(Type.CONTEXT)
public class SantanderSynchronizeJobProviderKontoauszug extends AbstractMahoeSynchronizeJobProviderKontoauszug {

    /**
     * Standardkonstruktor
     */
    public SantanderSynchronizeJobProviderKontoauszug() {
        super();
    }

    /**
     * @see AbstractMahoeSynchronizeJobProvider#getJobs()
     */
    @Override
    protected List<Class<? extends SynchronizeJob>> getJobs() {

        final List<Class<? extends SynchronizeJob>> list = new ArrayList<>(1);
        list.add(SantanderSynchronizeJobKontoauszug.class);
        return list;
    }

    /**
     * @see AbstractMahoeSynchronizeJobProvider#getSupportedAccountTypes()
     */
    @Override
    protected Set<KontoType> getSupportedAccountTypes() {

        final Set<KontoType> set = new HashSet<>(1);
        // add supported account types
        set.add(KontoType.GIRO);
        set.add(KontoType.KREDITKARTE);
        return set;
    }

    /**
     * @see AbstractMahoeSynchronizeJobProvider#isBankSupported(Konto)
     */
    @Override
    protected boolean isBankSupported(final Konto account) throws RemoteException {

        return SantanderBank.BLZ.equals(account.getBLZ());
    }

}
