/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.assistant;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.assistant.AbstractAccountMigrator;
import de.frankmuenster.mahoe.assistant.AccountMigrator;
import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.plugin.Plugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Die Migrator/Updataer fr alte TargoBank Konten
 *
 * @author Frank Mnster
 */
public class TargoBankAccountMigrator extends AbstractAccountMigrator implements AccountMigrator {

    /**
     * Der alte Plugin Name
     */
    private static final String OLD_PLUGIN_NAME = "hibiscus.script.targo"; //$NON-NLS-1$

    /**
     * Standardkonstruktor
     */
    public TargoBankAccountMigrator() {

        super();
    }

    /**
     * @throws ApplicationException
     * @see de.frankmuenster.mahoe.assistant.AccountMigrator#migrate(Plugin)
     */
    @Override
    public void migrate(final Plugin plugin) throws ApplicationException {

        int countMigrated = 0;

        if (TargoBankAccountMigrator.OLD_PLUGIN_NAME.equals(plugin.getManifest().getName())) {
            Logger.info("TargoBank-Migrator gestartet."); //$NON-NLS-1$
            try {
                final HBCIDBService hcbiService = Settings.getDBService();
                final DBIterator<Konto> accounts = hcbiService.createList(Konto.class);
                while (accounts.hasNext()) {
                    final Konto account = accounts.next();
                    if (TargoBank.BLZ.equals(account.getBLZ())
                            && !MahoeSynchronizeBackend.BACKEND_NAME.equals(account.getBackendClass())) {
                        this.migrateAccount(account);
                        countMigrated++;
                    }
                }

            } catch (final RemoteException e) {
                Logger.error(I18N.tr("TargoBankAccountMigrator.searchAccountsFailed", e.getMessage()), e); //$NON-NLS-1$
            }

            if (countMigrated > 0) {
                final StatusBarMessage statusBarMsg = new StatusBarMessage(
                        I18N.tr("TargoBankAccountMigrator.migrationSuccessful"), //$NON-NLS-1$
                        StatusBarMessage.TYPE_SUCCESS);
                Application.getMessagingFactory().sendMessage(statusBarMsg);
            }

            // Plugin entfernen
            this.uninstallPlugin(plugin);
        }

    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractAccountMigrator#update(de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public void update(final Konto account) throws ApplicationException {

        try {
            if (TargoBank.BLZ.equals(account.getBLZ())) {
                // Ist TargoBank
                final String oldVersion = account.getMeta(MahoeSynchronizeBackend.KONTO_VERSION, "0"); //$NON-NLS-1$

                switch (oldVersion) { // NOPMD
                case "0": //$NON-NLS-1$
                    // Update to 1
                    Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "1")); //$NON-NLS-1$
                    final String cardNumner = account.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, StringUtils.EMPTY);
                    account.setMeta(MahoeSynchronizeBackend.CARD_ACCOUNT, cardNumner);
                    account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "1"); //$NON-NLS-1$
                case "1": //$NON-NLS-1$
                    Logger.info(I18N.tr(AbstractAccountMigrator.I18N_UPDATE_TO_VERSION, "2")); //$NON-NLS-1$
                    Utils.removePdfMeta(account);
                    account.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, "2"); //$NON-NLS-1$
                default:
                    break;
                }

            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AbstractAccountMigrator.updatedFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("AbstractAccountMigrator.updatedFailedMsg", e.getMessage()), e); //$NON-NLS-1$
        }

    }

    /**
     * Migriert ein TragoBank-Skript Konto nach Mahoe
     *
     * @param konto
     *            das evtl. zu migrierende Konto
     * @throws ApplicationException
     */
    private void migrateAccount(final Konto konto) throws ApplicationException {

        // Lese alte Einstellungen
        final TargoBankCreateInfo info = new TargoBankCreateInfo();

        try {
            info.setCardNumber(konto.getIban());
            info.setCardAccount(konto.getIban());
            info.setComment(StringUtils.EMPTY);
            info.setCompany(konto.getUnterkonto());
            info.setLogin(konto.getKundennummer());

            konto.setKontonummer(TargoBank.KK_KTO);
            konto.setIban(TargoBank.KK_IBAN);
            konto.setKundennummer(info.getLogin());
            konto.setUnterkonto(StringUtils.EMPTY);
            konto.setBLZ(TargoBank.BLZ);
            konto.setAccountType(KontoType.KREDITKARTE.getValue());
            konto.setFlags(Konto.FLAG_NONE);
            konto.setKommentar(info.getComment());

            konto.setBackendClass("de.frankmuenster.mahoe.MahoeSynchronizeBackend"); //$NON-NLS-1$
            konto.store();

            // Setze META Daten
            konto.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, TargoBank.KTO_VER_KREDIT);
            konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY, info.getCompany());
            konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, info.getCardNumber());

            // Sync-Options setzen
            final SynchronizeOptions sync = new SynchronizeOptions(konto);
            sync.setAutoSaldo(false);
            sync.setSyncAuslandsUeberweisungen(false);
            sync.setSyncKontoauszuege(true);
            sync.setSyncOffline(false);
            sync.setSyncSaldo(true);
            sync.setSyncSepaDauerauftraege(false);
            sync.setSyncSepaLastschriften(false);

        } catch (final RemoteException e) {
            Logger.error(I18N.tr("TargoBankAccountMigrator.migrationFailedLog"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("TargoBankAccountMigrator.migrationFailedMsg", e.getMessage()), e); //$NON-NLS-1$
        }
    }

}
