/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.pdfextractor;

import java.awt.Rectangle;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPage;
import de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPageFirst;

/**
 * TextAreas fr die erste Seite
 *
 * @author Frank Mnster
 */
public class TargoBankPdfPageFirst extends AbstractBankPDFPageFirst {

    /**
     * Erwartete X-Position des Buchungsdaten
     */
    private static final double BUCH_DATUM_X_POS = 116.2;

    /**
     * Standard constructor call super class.
     *
     * @throws IOException
     */
    public TargoBankPdfPageFirst() throws IOException {

        super();
    }

    /**
     * Define first page regions.
     *
     * @return the map
     * @see AbstractBankPDFPage#defineRegions
     */
    @Override
    protected Map<String, Rectangle> defineFirstPageRegions() {

        final Map<String, Rectangle> regions = new HashMap<>(); // NOPMD

        regions.put(AbstractBankPDFPage.RECHNUNGS_DATUM, AbstractBankPDFPage.getRectangleFrom(15.5, 10.3, 5.0, 0.6));
        regions.put(AbstractBankPDFPage.KARTEN_KONTO, AbstractBankPDFPage.getRectangleFrom(15.5, 11.1, 5.0, 0.6));
        regions.put(AbstractBankPDFPage.SALDO, AbstractBankPDFPage.getRectangleFrom(12.0, 14.7, 1.5, 0.5));
        // regions.put(AbstractBankPDFPage.KARTEN_INHABER,
        // AbstractBankPDFPage.getRectangleFrom(12.5, 4.3, 4.0, 0.6))
        // regions.put(AbstractBankPDFPage.FAELLIG_BETRAG,
        // AbstractBankPDFPage.getRectangleFrom(17.0, 8.3, 3.0, 0.8))
        // regions.put(AbstractBankPDFPage.FAELLIG_DATUM,
        // AbstractBankPDFPage.getRectangleFrom(17.0, 8.9, 3.0, 0.8))
        // regions.put(AbstractBankPDFPage.FAELLIG_IBAN,
        // AbstractBankPDFPage.getRectangleFrom(14.5, 27.8, 8.0, 0.3))
        // regions.put(AbstractBankPDFPage.FAELLIG_BIC,
        // AstractBankPDFPage.getRectangleFrom(14.5, 28.1, 8.0, 0.3))
        return regions;
    }

    /**
     * @see AbstractBankPDFPage#defineRegions
     */
    @Override
    protected Map<String, Rectangle> defineRegions() {

        final Map<String, Rectangle> regions = new HashMap<>(); // NOPMD
        regions.put(AbstractBankPDFPage.KAUF_DATUM, AbstractBankPDFPage.getRectangleFrom(2.6, 15.5, 1.4, 8.5));
        regions.put(AbstractBankPDFPage.BUCHUNGS_DATUM, AbstractBankPDFPage.getRectangleFrom(3.9, 15.5, 1.4, 8.5));
        regions.put(AbstractBankPDFPage.BESCHREIBUNG, AbstractBankPDFPage.getRectangleFrom(5.7, 15.5, 5.8, 8.5));
        regions.put(AbstractBankPDFPage.BETRAG, AbstractBankPDFPage.getRectangleFrom(11.8, 15.5, 3.8, 8.5));
        regions.put(AbstractBankPDFPage.FREMDWAEHRUNG, AbstractBankPDFPage.getRectangleFrom(15.8, 15.5, 2.1, 8.5));
        regions.put(AbstractBankPDFPage.FREMDW_BETRAG, AbstractBankPDFPage.getRectangleFrom(15.8, 15.5, 2.1, 8.5));
        regions.put(AbstractBankPDFPage.KURS, AbstractBankPDFPage.getRectangleFrom(18.2, 15.5, 1.9, 8.5));
        return regions;
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPage#getXPos()
     */
    @Override
    protected double getXPos() {

        return TargoBankPdfPageFirst.BUCH_DATUM_X_POS;
    }

}
