
package de.frankmuenster.mahoe.banks.targobank.pdfextractor;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map.Entry;
import java.util.Set;

import de.frankmuenster.mahoe.pdfextractor.AbstractTransactionCreator;
import de.frankmuenster.mahoe.pdfextractor.PdfBuchung;
import de.frankmuenster.mahoe.pdfextractor.PdfEintrag;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * Implementierung des TransactionCreator der TargoBank
 *
 * @author Frank Mnster
 */
public class TargoBankTransactionCreator extends AbstractTransactionCreator {

    /**
     * Interest on debt
     */
    private static final String INTEREST_ON_DEBT = "Sollzinsen"; //$NON-NLS-1$

    /**
     * Interest on late payment
     */
    private static final String INTEREST_ON_LATEPAYMENTS = "Verzugszinsen"; //$NON-NLS-1$

    /**
     * Interest on deposit
     */
    private static final String INTEREST_ON_DEPOSIT = "Habenzinsen"; //$NON-NLS-1$

    /**
     * List with transactions
     */
    private final List<PdfEintrag> buchungsDaten;

    /**
     * @param buchungsDaten
     * @param lineHeigh
     */
    public TargoBankTransactionCreator(final List<PdfEintrag> buchungsDaten, final double lineHeigh) {
        super(buchungsDaten, lineHeigh);

        this.buchungsDaten = buchungsDaten;

        // shit need to redo the rows her
        recalulateRows();

        final Set<Entry<Long, Integer>> taNumberSet = this.taNumbers.entrySet();
        for (final Entry<Long, Integer> taNumber : taNumberSet) {
            Logger.debug(taNumber.getKey() + ":" + taNumber.getValue()); //$NON-NLS-1$
        }
    }

    @Override
    public List<PdfBuchung> getTansactions() {

        // remove transaction for debts with 0.00 EUR

        final Iterator<PdfBuchung> iter = this.transactions.iterator();

        while (iter.hasNext()) {
            final PdfBuchung pdfBuchung = iter.next();
            final String zweck = pdfBuchung.getZweck();

            Logger.debug(zweck);
            Logger.debug(TargoBankTransactionCreator.INTEREST_ON_DEBT);
            Logger.debug(TargoBankTransactionCreator.INTEREST_ON_DEBT);
            Logger.debug(TargoBankTransactionCreator.INTEREST_ON_DEBT);

            if (zweck.startsWith(TargoBankTransactionCreator.INTEREST_ON_DEBT)
                    || zweck.startsWith(TargoBankTransactionCreator.INTEREST_ON_LATEPAYMENTS)
                    || zweck.startsWith(TargoBankTransactionCreator.INTEREST_ON_DEPOSIT)) {

                if (pdfBuchung.getBetrag().compareTo(BigDecimal.ZERO) == 0) { // NOPMD
                                                                              // on
                                                                              // 20.06.21,
                                                                              // 19:20
                    iter.remove();
                }
            }
        }

        return this.transactions;
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractTransactionCreator#getDateFormat()
     */
    @Override
    protected SimpleDateFormat getDateFormat() {

        return new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$

    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractTransactionCreator#getNumberFormat()
     */
    @Override
    protected NumberFormat getNumberFormat() {

        return NumberFormat.getInstance(Locale.GERMAN);

    }

    @Override
    protected Integer getRow(final double posY) {

        Integer row = null;

        // loop of all transactions to find the row
        for (int i = 0; i < this.buchungsDaten.size(); i++) {
            Double dateRow = this.buchungsDaten.get(i).getPositionY();
            final Double sndRow = dateRow - this.lineHeigh;
            // move 1.5 points up (plus) as some values are shifted about 1.4
            // points
            dateRow += 1.5;
            Logger.debug(String.format("posY: %f --> [%f-%f] ", posY, dateRow, sndRow)); //$NON-NLS-1$

            if (dateRow >= posY && posY >= sndRow) {
                row = i;
                break;
            }
        }

        Logger.debug(String.format("Umsatzzeile: %d", row)); //$NON-NLS-1$

        return row;
    }

    private void recalulateRows() {

        this.taNumbers.clear();
        this.transactions.clear();

        for (final PdfEintrag buchungsDatum : this.buchungsDaten) {
            try {
                final Date bDatum = this.sdf.parse(buchungsDatum.getWort());
                final PdfBuchung buchung = new PdfBuchung(); // NOPMD
                buchung.setBuchungsDatum(bDatum);
                this.transactions.add(buchung);

            } catch (final ParseException e) {
                Logger.error(I18N.tr("AbstractTransactionCreator.bookingDateParseError", e.getMessage())); //$NON-NLS-1$
            }
        }
    }

}
