/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.mahoe.messages.MahoeMessage;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.WebUtils;
import de.frankmuenster.mahoe.web.AbstractWebSite;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * List und Speichert Bank-Nachrichten
 *
 * @author Frank Mnster
 */
public class TargoBankMessages extends AbstractBankMessages<WebDriver> {

    /**
     *
     */
    private static final String NO_MESSAGES = "Sie haben keine Mitteilung."; //$NON-NLS-1$

    /**
     * Erstellt ein neues Objekt
     *
     * @param webclient
     */
    public TargoBankMessages(final WebDriver webClient) {
        super(webClient);
    }

    /**
     * @see de.frankmuenster.mahoe.messages.InterfaceBankMessages#markAsRead(de.frankmuenster.mahoe.messages.MahoeMessage)
     */
    @Override
    public void markAsRead(final MahoeMessage message) throws ApplicationException {
        // Wir bei der TargoBank nicht bentigt, da wir auf den Link 'Klicken'.

    }

    /**
     * @see de.frankmuenster.mahoe.messages.AbstractBankMessages#readMessages(java.util.Date)
     * @throws WebSiteException
     * @throws ApplicationException
     */
    @Override
    public void readMessages(final Date startDate) throws WebSiteException, ApplicationException {

        Date sDate;

        // Startdatum ermitteln
        if (startDate == null) {
            final Calendar cal = Calendar.getInstance();
            cal.clear(Calendar.HOUR);
            cal.clear(Calendar.MINUTE);
            cal.clear(Calendar.SECOND);
            cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) - 1);
            sDate = cal.getTime();
        } else {
            sDate = startDate;
        }
        final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm", //$NON-NLS-1$
                Locale.GERMAN);

        try {
            // open the messages page
            this.webclient.get(TargoBank.URL_MESSAGES);

            // get the message table
            final String tableXPatch = "/html/body/div[3]/div[2]/div/div/section/article/div/div/form/div[1]/div[2]/div[2]/div[2]/div/table/tbody"; //$NON-NLS-1$ // NOPMD 
            final WebElement tdBody = this.webclient.findElement(By.xpath(tableXPatch));

            final List<WebElement> rows = tdBody.findElements(By.tagName(WebUtils.HTML_TAG_TR));
            final int rowsCount = rows.size();
            // /html/body/div[3]/div[2]/div/div/section/article/div/div/form/div[1]/div[2]/div[2]/div[2]/div/table/tbody/tr[1]
            for (int i = 1; i <= rowsCount; i++) {
                // get row by xpath completely new
                final String rowXPath = tableXPatch + "/tr[" + i + "]"; //$NON-NLS-1$ //$NON-NLS-2$
                Logger.debug("Row XPath: " + rowXPath); //$NON-NLS-1$
                final WebElement row = this.webclient.findElement(By.xpath(rowXPath));

                if (row.getText().contains(TargoBankMessages.NO_MESSAGES)) {
                    Logger.info(I18N.tr("TargoBankMessages.noNewMessages"));//$NON-NLS-1$
                    break;
                }
                // get date
                final String dateString = row.findElement(By.xpath("td[4]")).getText().trim() //$NON-NLS-1$
                        .replaceAll(AbstractWebSite.NON_BREAK_SPACE, StringUtils.EMPTY);
                Logger.debug(I18N.tr("TargoBankAccountStatementsPage.billingDate", dateString)); //$NON-NLS-1$
                final Date date = sdf.parse(dateString);
                if (date.after(sDate)) {
                    final WebElement subjectAnchor = row.findElement(By.xpath("td[3]/*/a|td[3]/a")); //$NON-NLS-1$
                    final String subject = subjectAnchor.getText();
                    if (!AbstractBankMessages.existsMessage(TargoBank.BLZ, date, subject)) {
                        // Nachricht in Liste einfgen
                        this.addMessage(subjectAnchor, date, subject);
                    }
                }
            }
        } catch (final NoSuchElementException e) {
            Logger.error(I18N.tr("TargoBankMessages.openMessagePageFailed"), e); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("TargoBankMessages.openMessagePageFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.PAGE_NOT_FOUND, e);
        } catch (final ParseException e) {
            Logger.error(I18N.tr("TargoBankMessages.parseDateFailed"), e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("TargoBankMessages.parseDateFailed"), e); //$NON-NLS-1$
        }

    }

    /**
     * Fgt die Bank-Nachricht der MahoeNachrichten Liste hinzu.
     *
     * @param subjectAnchor
     *            der Link zur Nachrichten Datei Seite
     * @param sdf
     * @param date
     *            das Datum
     * @param subject
     *            der Betreff
     * @throws WebSiteException
     *             falls der Link nicht geffnet werden kann
     * @throws ParseException
     */
    private void addMessage(final WebElement subjectAnchor, final Date date, final String subject)
            throws WebSiteException, ParseException {

        Logger.debug("Lese Nachricht"); //$NON-NLS-1$

        String msgString = StringUtils.EMPTY;

        try {

            // open message
            subjectAnchor.click();

            final WebElement tdBody = this.webclient.findElement(By.xpath(
                    "/html/body/div[3]/div[2]/div/div/section/article/div/div/form/div/div[2]/div[2]/div[2]/fieldset/table/tbody")); //$NON-NLS-1$
            final List<WebElement> rows = tdBody.findElements(By.xpath(WebUtils.HTML_TAG_TR));
            for (final WebElement row : rows) {
                final String rowContent = row.findElement(By.xpath(WebUtils.HTML_TAG_TD1)).getText().trim();
                if (rowContent.startsWith("Nachricht")) { //$NON-NLS-1$
                    msgString = row.findElement(By.xpath(WebUtils.HTML_TAG_TD + "[2]")).getText(); //$NON-NLS-1$
                    break;
                }

            }

            final MahoeMessage message = new MahoeMessage(TargoBank.BLZ, date, subject, msgString);
            this.messages.add(message);

            // go back to messages window
            final WebElement back = this.webclient.findElement(By.id("B")); //$NON-NLS-1$
            back.click();

        } catch (final NoSuchElementException e) {
            Logger.error(I18N.tr("TargoBankMessages.openMessageDetailsFailed"), e); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("TargoBankMessages.openMessageDetailsFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.PAGE_NOT_FOUND, e);
        }

    }
}
