/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web;

import java.math.BigDecimal;
import java.time.LocalDate;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractWebAccountInformation;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Bean mit Kontoinformationen von der Webseite
 *
 * @author Frank Mnster
 * @version 1.0
 */
public class TargoBankWebAccountInformation extends AbstractWebAccountInformation {

    /**
     * Das Kartenkonto
     */
    private String cardAccount;
    /**
     * Der Kartenanbieter
     */
    private String cardCompany;

    /**
     * Die Kartennummer
     */
    private String cardNumber;

    /**
     * Nchster Flligkeitstermin
     */
    private LocalDate nextPayDay;

    /**
     * Der Verfgungsrahmen
     */
    private BigDecimal verfuegungsrahmen;

    /**
     * Standardkonstruktor
     */
    public TargoBankWebAccountInformation() {

        super();
    }

    /**
     * @return the cardAccount
     */
    public String getCardAccount() {

        return this.cardAccount;
    }

    /**
     * @return the cardCompany
     */
    public String getCardCompany() {

        return this.cardCompany;
    }

    /**
     * @return the cardNumber
     */
    public String getCardNumber() {

        return this.cardNumber;
    }

    /**
     * @return the nextPayDay
     */
    public LocalDate getNextPayDay() {

        return this.nextPayDay;
    }

    /**
     * @return the verfuegungsrahmen
     *
     * @deprecated since 2.6.0, verfuegunsrahmen don't exist anymore on the page
     */
    @Deprecated
    public BigDecimal getVerfuegungsrahmen() {

        return this.verfuegungsrahmen;
    }

    /**
     * @param cardAccount
     *            the cardAccount to set
     */
    public void setCardAccount(final String cardAccount) {

        this.cardAccount = cardAccount;
    }

    /**
     * @param cardCompany
     *            the cardCompany to set
     */
    public void setCardCompany(final String cardCompany) {

        this.cardCompany = cardCompany;
    }

    /**
     * @param cardNumber
     *            the cardNumber to set
     */
    public void setCardNumber(final String cardNumber) {

        this.cardNumber = cardNumber;
    }

    /**
     * @param nextPayDay
     *            the nextPayDay to set
     */
    public void setNextPayDay(final LocalDate nextPayDay) {

        this.nextPayDay = nextPayDay;
        this.updateComment();

    }

    /**
     * @param verfuegungsrahmen
     *            the verfuegungsrahmen to set
     */
    public void setVerfuegungsrahmen(final BigDecimal verfuegungsrahmen) {

        this.verfuegungsrahmen = verfuegungsrahmen;
        this.updateComment();
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebAccountInformation#saveBankSpecificInfo(de.willuhn.util.ProgressMonitor)
     */
    @Override
    protected void saveBankSpecificInfo(final ProgressMonitor monitor) throws ApplicationException {

        // Noch nix zu tun
    }

    private void updateComment() {

        final String rahmen = this.verfuegungsrahmen == null ? StringUtils.EMPTY : this.verfuegungsrahmen.toString();
        final String payDay = this.nextPayDay == null ? StringUtils.EMPTY
                : Utils.convertLocalDateToString(this.nextPayDay);

        final String kommentar = I18N.tr("TargoBankSynchronizeJobKontoauszug.comment", //$NON-NLS-1$
                rahmen, payDay);

        this.setComment(kommentar);

    }
}
