/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web; // NOPMD

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.banks.targobank.pdfextractor.TargoBankPdfExtractor;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankDetailsPage;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankLoginPage;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankPostBoxPage;
import de.frankmuenster.mahoe.banks.targobank.web.webpages.TargoBankTransactionsPage;
import de.frankmuenster.mahoe.pdfextractor.InvalidCardNumberException;
import de.frankmuenster.mahoe.pdfextractor.PdfBuchung;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.PasswdCache;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractSeleniumWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.LoginPage;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Stellt das Webseiten-Interface dar.
 *
 * @author Frank Mnster
 */
public final class TargoBankWebSite extends AbstractSeleniumWebSite<TargoBankWebAccountInformation, TargoBankMessages> {

    /**
     * Unsere BceeWebSite instance
     */
    private static TargoBankWebSite webSite;

    /**
     * Das Konto
     */
    private final Konto konto;

    /**
     * Das Login/die Kundenkennung
     */
    private final String kundenkennung;

    /**
     * Returns the account sheet as byte array
     *
     * @param document
     *            the message from the postbox with download link inside
     * @param webClient
     * @return the account sheet as byte array
     * @throws WebSiteException
     *             if the download of the account sheet failed
     */
    // public static byte[] getAccountSheetAsByteArray(final Pair<Date,
    // WebElement> document, final WebDriver webClient)
    // throws WebSiteException {
    //
    // // extract link element and download as byte array
    // Logger.debug(document.getKey().toString() + ":" +
    // document.getValue().getText()); //$NON-NLS-1$
    // final WebElement pdflink = document.getValue();
    //
    // // scroll into view
    // WebUtilsSelenium.scrollTo(webClient, pdflink);
    //
    // byte[] pdf = WebUtilsSelenium.getSeleniumPdfAsByteArray(webClient,
    // pdflink);
    // // check if file has content
    // if (pdf.length == 0) {
    // // Retry download
    // pdf = WebUtilsSelenium.getSeleniumPdfAsByteArray(webClient, pdflink);
    // }
    // if (pdf.length == 0) {
    // throw new
    // WebSiteException(I18N.tr("SantanderWebSite.atLeastOneDownloadFailed"),
    // //$NON-NLS-1$
    // WebSiteExceptionTypes.FILE_DOWNLOAD_FAILED);
    // }
    //
    // return pdf;
    //
    // }

    /**
     * Liefert die Instance der WebSeite
     *
     * @param konto
     * @return die WebSeite
     * @throws WebSiteException
     * @throws ApplicationException
     */
    public static TargoBankWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {

        if (TargoBankWebSite.webSite == null) { // NOPMD
            TargoBankWebSite.webSite = new TargoBankWebSite(konto);
        } else {
            // Ist das Konto dasselbe
            try {
                if (TargoBankWebSite.webSite.konto.equals(konto)) {
                    // Aktualisiere Startdatum der Umsatzabfrage
                    final Date lastTransacDate = Utils.getLastUmsatzDate(konto,
                            TargoBankWebSite.webSite.getStartdate());
                    TargoBankWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
                } else {
                    TargoBankWebSite.webSite.logoff();
                    TargoBankWebSite.webSite = new TargoBankWebSite(konto);
                }
            } catch (final RemoteException e) {
                final String msg = I18N.tr("TargoBankWebSite.readAccountInfoFailed"); //$NON-NLS-1$
                Logger.error(msg, e);
                throw new ApplicationException(msg, e);
            }
        }

        return TargoBankWebSite.webSite;
    }

    /**
     * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der
     * TargoBank.
     *
     * @param konto
     * @param login
     * @param cardCompany
     * @param cardAccount
     * @param cardNumber
     * @throws WebSiteException
     * @throws ApplicationException
     */
    private TargoBankWebSite(final Konto konto) throws WebSiteException, ApplicationException {

        super(konto);
        this.konto = konto;

        String cardAccount;
        String cardNumber;
        String cardCompany;
        String loginName;

        try {
            loginName = konto.getKundennummer();
            if (StringUtils.isEmpty(loginName)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.loginMissing")); //$NON-NLS-1$
            }
            cardAccount = konto.getMeta(MahoeSynchronizeBackend.CARD_ACCOUNT, null);
            if (StringUtils.isEmpty(cardAccount)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardAccountMissing")); //$NON-NLS-1$
            }
            cardNumber = konto.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, null);
            if (StringUtils.isEmpty(cardNumber)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardNumberMissing")); //$NON-NLS-1$
            }
            cardNumber = cardNumber.replaceAll("\\s+", StringUtils.EMPTY); //$NON-NLS-1$
            cardCompany = konto.getMeta(MahoeSynchronizeBackend.CARD_COMPANY, null);
            if (StringUtils.isEmpty(cardCompany)) {
                throw new ApplicationException(I18N.tr("TargoBankWebSite.cardCompanyMissing")); //$NON-NLS-1$
            }

        } catch (final RemoteException e) {
            final String msg = I18N.tr("TargoBankWebSite.readAccountInfoFailed"); //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        }

        this.kundenkennung = loginName;
        this.accountInfo.setCardCompany(cardCompany);
        this.accountInfo.setCardNumber(cardNumber);
        this.accountInfo.setCardAccount(cardAccount);

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        Logger.info(I18N.tr("AbstractWebSite.transferNotSupported")); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public TargoBankWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        final TargoBankDetailsPage detailsPage = new TargoBankDetailsPage(this.webClient, this.accountInfo);

        this.accountInfo.setSaldo(detailsPage.getSaldo());
        this.accountInfo.setVerfuegungsrahmen(detailsPage.getCreditLimit());
        this.accountInfo.setSaldoAvailable(detailsPage.getAvailableAmount());
        this.accountInfo.setNextPayDay(detailsPage.getNextSubtraction());

        return this.accountInfo;
    }

    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        final TargoBankAccountStatementFinder accountSheets = new TargoBankAccountStatementFinder(this.webClient,
                accountInfo);
        accountSheets.findStatements();

        return accountSheets.getSheets();
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        // Initialise the list
        final List<Umsatz> transactions = new ArrayList<>();

        // get card account number
        final String accountNumber = this.accountInfo.getCardAccount();

        // get a new PoxBoxPage
        final TargoBankPostBoxPage postBoxPage = new TargoBankPostBoxPage(this.webClient, this.accountInfo);

        while (postBoxPage.hasNextAccountSheet()) {
            try {
                // get the next document link
                final Pair<Date, WebElement> document = postBoxPage.nextAccountSheet();

                // check date
                if (document.getKey().before(this.accountInfo.getLastTransactionDate())) {
                    Logger.debug(I18N.tr("TargoBankWebSiteTransactionsImpl.reachedLastPdf")); //$NON-NLS-1$
                    break;
                }
                // get the PDF as byte array
                final byte[] pdf = super.getAccountSheetAsByteArray(document, this.webClient);

                // read Transactions from PDF
                final TargoBankPdfExtractor extractor = new TargoBankPdfExtractor(pdf, accountNumber); // NOPMD
                Logger.debug(I18N.tr("AbstractWebSite.extractTransactions", accountNumber)); //$NON-NLS-1$
                final List<PdfBuchung> buchungenPDF = extractor.getExtract();

                // Add all transactions
                transactions.addAll(TransactionSaver.getTransactions(buchungenPDF));

            } catch (final InvalidCardNumberException e) {
                Logger.warn(I18N.tr("WebSite.notMatchingCreditcradNumber") + e.getLocalizedMessage()); //$NON-NLS-1$
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("WebSite.errorReadingBookedCreditcardTransactions"), e); //$NON-NLS-1$
                throw new ApplicationException(I18N.tr("WebSite.errorReadingBookedCreditcardTransactions"), e); //$NON-NLS-1$
            }
        }

        return transactions;

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        // final TargoBankWebSiteTransactionsImpl transImpl = new
        // TargoBankWebSiteTransactionsImpl(this.webClient,
        // this.accountInfo);

        // get a new TransactionsPage
        final TargoBankTransactionsPage transactionsPage = new TargoBankTransactionsPage(this.webClient,
                this.accountInfo);

        // return the found pre-book transactions
        return transactionsPage.getTransactions(this.accountInfo.getLastTransactionDate());
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        return new TargoBankLoginPage(this.webClient).isLoggedIn();

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], String...)
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        // first check if driver available
        // TODO: check if necessary
        this.checkSeleniumDriverSet(this.webClient);

        if (loginParams.length != 0) {
            throw new WebSiteException(I18N.tr("TargoBankWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR);
        }

        try {
            final LoginPage loginPage = new TargoBankLoginPage(this.webClient);
            loginPage.login(this.kundenkennung, password);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)
                    || e.getType().equals(WebSiteExceptionTypes.LOGIN_ERROR)) {
                // Passwort zurcksetzen
                PasswdCache.revoke(TargoBank.NAME, this.kundenkennung);
            }
            Arrays.fill(password, ' ');
            throw e;
        }
        Arrays.fill(password, ' ');

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() throws WebSiteException {

        if (!isStillLoggedIn()) {
            return;
        }

        // Stay logged in for debug purpose
        if (!this.accountInfo.isStayLoggedIn()) {
            // call log off URL
            this.webClient.get(TargoBank.URL_LOGOFF);

            // check if log off was successful
            final String title = this.webClient.getTitle();
            if (title.startsWith("Online-Banking Logout")) { //$NON-NLS-1$
                Logger.info(I18N.tr("TargoBankWebSite.logoffSuccessful")); //$NON-NLS-1$
            } else {
                throw new WebSiteException(I18N.tr("TargoBankWebSite.logoffFailed"), //$NON-NLS-1$
                        WebSiteExceptionTypes.LOGOFF_FAILED);
            }
        }

        // finally save the cookies
        saveCookies(TargoBank.COOKIE_FILE);
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // set start date to August 2020 (new format of teh PDF account sheet)
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.set(2020, 7, 1);
        return startCalendar;
    }

}
