/**
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web.webpages;

import java.util.List;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebDriverException;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.banks.targobank.web.TargoBankWebAccountInformation;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.utils.WebUtils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.AbstractAccountPage;
import de.willuhn.logging.Logger;

/**
 * The account page the account provided for a given account
 *
 * @author papa
 *
 */
public class TargoBankAccountPage extends AbstractAccountPage<TargoBankWebAccountInformation> {

    /**
     * Creates a new AccountPage for account provided within the given account
     * information
     *
     * @param driver
     *            the Selenium web driver to use
     * @param accountInfo
     *            the account information object
     */
    public TargoBankAccountPage(final WebDriver driver, final TargoBankWebAccountInformation accountInfo) {
        super(driver, accountInfo);

    }

    @Override
    public void open() throws WebSiteException {

        this.driver.get(TargoBank.URL_KTO_UEBERSICHT);

        if (!elementExist(By.id("I0:A1:al"))) { //$NON-NLS-1$
            throw new WebDriverException("Kontenbersicht konnte nicht geffnet werden.");

        }
        WebElement link = null;

        final WebElement tableDiv = this.driver.findElement(By.id("I0:A1:al")); //$NON-NLS-1$
        final List<WebElement> tables = tableDiv.findElements(By.tagName(WebUtils.HTML_TAG_TABLE));

        for (final WebElement table : tables) {
            final WebElement tableBody = table.findElement(By.tagName(WebUtils.HTML_TAG_TBODY));

            final List<WebElement> rows = tableBody.findElements(By.tagName(WebUtils.HTML_TAG_TR));

            for (final WebElement row : rows) {

                Logger.debug(row.toString());

                if (elementExist(By.tagName(WebUtils.HTML_TAG_TD))) {
                    final List<WebElement> cells = row.findElements(By.tagName(WebUtils.HTML_TAG_TD));
                    if (!cells.isEmpty() && isAccount(cells.get(0))) {
                        link = cells.get(0);
                        break;
                    }
                }
            }
        }

        // If link is null the account has not been found
        if (link == null) {
            throw new WebSiteException("KreditkartenKonto nicht gefunden", WebSiteExceptionTypes.ACCOUNT_NOT_FOUND);
        }
        // open account
        link.click();

        final WebElement check = this.driver.findElement(By.id("F12:T:ttlBlockRoot")); //$NON-NLS-1$

        Logger.debug(check.getText());
        if (!check.getText().contains("Kreditkartenbersicht")) { //$NON-NLS-1$
            throw new WebSiteException("Fehler beim ffnen der Kreditkartenbersicht",
                    WebSiteExceptionTypes.WEBSITE_NOT_REACHABLE);

        }

    }

    private boolean isAccount(final WebElement cell) {

        final String cellText = cell.getText().trim();
        Logger.debug("Zelle1: " + cellText); //$NON-NLS-1$

        final String linkName = this.accountInfo.getCardCompany() + "\n" //$NON-NLS-1$
                + Utils.maskCardNummer(this.accountInfo.getCardNumber()); // $NON-NLS-1$
        Logger.debug(linkName);

        return cellText.endsWith(Utils.maskCardNummer(this.accountInfo.getCardNumber()));

    }

}
