/**
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web.webpages;

import java.math.BigDecimal;
import java.time.LocalDate;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.targobank.web.TargoBankWebAccountInformation;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.utils.WebUtils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.webpages.DetailsPage;
import de.willuhn.logging.Logger;

/**
 * Instance of the TargoBank details page to retrieve account details
 *
 * @author papa
 *
 */
public class TargoBankDetailsPage extends TargoBankAccountPage implements DetailsPage {

    /** the saldo */
    private BigDecimal saldo;

    /** credit Limit */
    private BigDecimal creditLimit;

    /** date of the last transaction */
    private LocalDate lastTransaction;

    /** date of next subtraction */
    private LocalDate nextSubtraction;

    /** the available amount */
    private BigDecimal availableAmount;

    /**
     * Creates a new details page for a TargoBank credit card account. Available
     * fields are pre-filled.
     *
     * @param driver
     * @param accountInfo
     * @throws WebSiteException
     */
    public TargoBankDetailsPage(final WebDriver driver, final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        super(driver, accountInfo);

        // open account in super class
        super.open();

        // get the details in fill variables
        setDetails();

    }

    @Override
    public BigDecimal getAvailableAmount() throws WebSiteException {
        return this.availableAmount;
    }

    @Override
    public BigDecimal getCreditLimit() throws WebSiteException {
        return this.creditLimit;
    }

    @Override
    public LocalDate getLastTransaction() throws WebSiteException {
        return this.lastTransaction;
    }

    @Override
    public LocalDate getNextBill() throws WebSiteException {
        throw new UnsupportedOperationException("NextBill wird bei der TargoBank nicht angezeigt");
    }

    @Override
    public LocalDate getNextSubtraction() throws WebSiteException {
        return this.nextSubtraction;
    }

    @Override
    public String getProductDescription() throws WebSiteException {
        throw new UnsupportedOperationException("ProductDescription wird bei der TargoBank nicht angezeigt");
    }

    @Override
    public BigDecimal getSaldo() throws WebSiteException {
        return this.saldo;
    }

    private void setDetails() {

        final WebElement tableBody = this.driver.findElement(By.id("F12:expContent")) //$NON-NLS-1$
                .findElement(By.tagName(WebUtils.HTML_TAG_TBODY));

        // availableAmount
        String text = tableBody.findElement(By.xpath("./tr[3]/td[1]")).getText(); //$NON-NLS-1$
        Logger.debug(String.format("Verfgbarer Betrag Text: %s", text)); //$NON-NLS-1$
        this.availableAmount = Utils.convertStringToBigDecimal(text);

        // creditLimit
        text = tableBody.findElement(By.xpath("./tr[2]/td[1]")).getText(); //$NON-NLS-1$
        Logger.debug(String.format("Verfgungsrahmen Text: %s", text)); //$NON-NLS-1$
        this.creditLimit = Utils.convertStringToBigDecimal(text);

        // lastTransaction
        text = tableBody.findElement(By.xpath("./tr[5]/td[1]")).getText().trim(); //$NON-NLS-1$
        Logger.debug(String.format("Datum der letzten Zahlung Text: >%s<", text)); //$NON-NLS-1$
        this.lastTransaction = Utils.convertTextToLocalDate(text, "dd.MM.yyyy"); //$NON-NLS-1$

        // nextSubtraction
        text = tableBody.findElement(By.xpath("./tr[3]/td[2]")).getText().trim(); //$NON-NLS-1$
        Logger.debug(String.format("Nchster Flligkeitstermin Text: %s", text)); //$NON-NLS-1$
        this.nextSubtraction = Utils.convertTextToLocalDate(text, "dd.MM.yyyy"); //$NON-NLS-1$

        // saldo
        text = tableBody.findElement(By.xpath("./tr[1]/td[1]")).getText(); //$NON-NLS-1$
        Logger.debug(String.format("Saldo Text: %s", text)); //$NON-NLS-1$
        this.saldo = Utils.convertStringToBigDecimal(text);

    }

}
