/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web.webpages;

import java.rmi.RemoteException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.targobank.web.TargoBankWebAccountInformation;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.utils.WebUtils;
import de.frankmuenster.mahoe.web.AbstractWebSite;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.webpages.TransactionsPage;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;

/**
 * The page with all listed transactions These are the booked transactions for
 * giro account and the pre-booked for an credit card account.
 *
 *
 * @author Frank Mnster
 * @since 2.6.0
 *
 */
public class TargoBankTransactionsPage extends TargoBankAccountPage implements TransactionsPage {

    /**
     * Will create a new instance of the page
     *
     * @param driver
     * @param accountInfo
     * @throws WebSiteException
     */
    public TargoBankTransactionsPage(final WebDriver driver, final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        super(driver, accountInfo);

        super.open();

    }

    /**
     * Returns all transactions on current page till the given date
     *
     * @param Date
     *            lastTransactionDate the of the last transactions to give back
     * @throws WebSiteException
     */
    @Override
    public List<Umsatz> getTransactions(final Date lastTransactionDate) throws WebSiteException {

        final List<Umsatz> transactions = new ArrayList<>();

        // check if transaction table exists
        if (this.driver.findElements(By.id("F14:expContent")).isEmpty()) { //$NON-NLS-1$
            return transactions; // NOPMD on 20.06.21, 19:14
        }

        final WebElement tableBody = this.driver.findElement(By.id("F14:expContent")) //$NON-NLS-1$
                .findElement(By.tagName(WebUtils.HTML_TAG_TBODY));

        final List<WebElement> rows = tableBody.findElements(By.tagName(WebUtils.HTML_TAG_TR));

        for (final WebElement row : rows) {

            try {

                Logger.debug(row.toString());

                // date
                final String date = row.findElement(By.xpath("./td[1]")).getText(); //$NON-NLS-1$
                Logger.debug(String.format("Datum: %s", date)); //$NON-NLS-1$

                // details
                final String details = row.findElement(By.xpath("./td[2]")).getText().trim(); //$NON-NLS-1$
                Logger.debug(String.format("Details: %s", details)); //$NON-NLS-1$

                // city
                final String city = row.findElement(By.xpath("./td[3]")).getText().trim(); //$NON-NLS-1$
                Logger.debug(String.format("Stadt: %s", city)); //$NON-NLS-1$

                // country
                final String country = row.findElement(By.xpath("./td[4]")).getText().trim(); //$NON-NLS-1$
                Logger.debug(String.format("Land: %s", country)); //$NON-NLS-1$

                // amount
                final String amount = row.findElement(By.xpath("./td[5]")).getText(); //$NON-NLS-1$
                Logger.debug(String.format("Betrag: %s", amount)); //$NON-NLS-1$

                // foreign amount
                final String foreignAmount = row.findElement(By.xpath("./td[6]")).getText().trim() //$NON-NLS-1$
                        .replaceAll(AbstractWebSite.NON_BREAK_SPACE, StringUtils.EMPTY);
                Logger.debug(String.format("Fremdwhrungsbetrag: %s", foreignAmount)); //$NON-NLS-1$

                // Create new transaction
                final HBCIDBService dbService = Settings.getDBService();
                Umsatz umsatz;
                umsatz = dbService.createObject(Umsatz.class, null);

                // Date
                final Date datum = Utils.convertTextToDate(date, "dd.MM.yyyy"); //$NON-NLS-1$
                umsatz.setDatum(datum);
                umsatz.setValuta(datum);

                // Purpose text, composed out of details, city and country
                final String zweck = details + " " + city + " " //$NON-NLS-1$ //$NON-NLS-2$
                        + country;
                Utils.loadUmsatz(umsatz, zweck);

                // Amount
                umsatz.setBetrag(Utils.convertStringToDouble(amount));

                // Foreign amount
                if (StringUtils.isNotEmpty(foreignAmount)) {
                    umsatz.setKommentar(
                            I18N.tr("TargoBankWebSiteTransactionsImpl.foreignCurrencySaldo", foreignAmount)); //$NON-NLS-1$
                }

                // mark as pre-booked transaction
                umsatz.setFlags(Umsatz.FLAG_NOTBOOKED);

                // add transaction to list
                transactions.add(umsatz);

            } catch (final RemoteException | ParseException e) {
                Logger.error(I18N.tr("TargoBankWebSiteTransactionsImpl.createUmsatzError", e.getMessage()), e); //$NON-NLS-1$
            }
        }
        return transactions;
    }

}
