/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.parts;

import java.rmi.RemoteException;

import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.mahoe.gui.controller.AbstractSettingsController;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.ColumnLayout;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Abstract class for a settings part. Provides already an account chooser, the
 * type of a chosen account and information about account statement downloading
 *
 * @author Frank Mnster
 *
 * @param <C>
 *            teh controller to handle the data of the part
 */
public abstract class AbstractSettingsPart<C extends AbstractSettingsController<?>> implements Part {

    /**
     *
     */
    protected static final String FIX_SPACE_TEXT = "         "; //$NON-NLS-1$

    /**
     * Our controller
     */
    protected final transient C control;

    /**
     * Creates a new settings part with the given controller
     *
     * @param control
     *            the controller to handle the data inside the settings part
     */
    protected AbstractSettingsPart(final C control) {
        this.control = control;
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        final ColumnLayout colMain = new ColumnLayout(parent, 2);

        final SimpleContainer header = new SimpleContainer(colMain.getComposite());
        header.addHeadline(I18N.tr("SantanderSettingsPart.accountSettingsHeadLine")); //$NON-NLS-1$
        header.addLabelPair(I18N.tr("SantanderSettingsPart.accountLbl"), this.control.getKonto()); //$NON-NLS-1$

        final ColumnLayout clContent = new ColumnLayout(parent, 2);
        final SimpleContainer left = new SimpleContainer(clContent.getComposite(), true, 1);
        final SimpleContainer right = new SimpleContainer(clContent.getComposite(), true);

        if (this.control.isDisplayAccountType()) {
            final SimpleContainer left1 = new SimpleContainer(left.getComposite(), false, 4);
            left1.addLabelPair(I18N.tr("SantanderSettingsPart.accountTypLbl"), this.control.getKontoTyp()); //$NON-NLS-1$
            left1.addText(AbstractSettingsPart.FIX_SPACE_TEXT, false);
            left1.addText(AbstractSettingsPart.FIX_SPACE_TEXT, false);
            left1.addText(AbstractSettingsPart.FIX_SPACE_TEXT, false);
        }
        if (this.control.isDisplayAccountStatement()) {
            final SimpleContainer left2 = new SimpleContainer(left.getComposite());
            left2.addHeadline(I18N.tr("SantanderSettingsPart.accoutSheetsHeadLine")); //$NON-NLS-1$

            left2.addLabelPair(I18N.tr("SantanderSettingsPart.saveAccountSheetsLbl"), this.control.getSavePDF()); //$NON-NLS-1$
            left2.addLabelPair(I18N.tr("SantanderSettingsPart.accStatementDownloadPeriod"), //$NON-NLS-1$
                    this.control.getStatementMaxDaysBack());
        }

        paintBankSpecific(left, right);

    }

    /**
     * Needs to be implemented by a concrete settings part. Can be used to
     * display/set bank/account specific information.
     *
     * @param left
     *            the left composite (2/3)
     * @param right
     *            the right composite (1/3)
     * @throws RemoteException
     */
    protected abstract void paintBankSpecific(SimpleContainer left, SimpleContainer right) throws RemoteException;

}