/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;

import de.frankmuenster.mahoe.gui.controller.SettingsController;
import de.frankmuenster.mahoe.gui.utils.AktienSettingsTabGroup;
import de.frankmuenster.mahoe.gui.utils.BosSettingsTabGroup;
import de.frankmuenster.mahoe.gui.utils.BshSettingsTabGroup;
import de.frankmuenster.mahoe.gui.utils.GeneralSettingsTabGroup;
import de.frankmuenster.mahoe.gui.utils.SantanderSettingsTabGroup;
import de.frankmuenster.mahoe.gui.utils.SettingsTabGroup;
import de.frankmuenster.mahoe.gui.utils.TargoBankSettingsTabGroup;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;

/**
 * View fr die Einstellungen
 *
 * @author Frank Mnster
 */
public class SettingsView extends AbstractView {

    /**
     * Liste aller TabGroups
     */
    private final transient List<SettingsTabGroup<SettingsController>> tabs = new ArrayList<>();

    /**
     * Der SWT TabFolder
     */
    private transient TabFolder folder;

    /**
     * Liefert eine neu Settings-View zurck und erstellt einen passenden
     * Controller
     */
    public SettingsView() {
        super();
    }

    /**
     * @see de.willuhn.jameica.gui.AbstractView#bind()
     */
    @Override
    public void bind() throws Exception { // NOPMD

        GUI.getView().setTitle(I18N.tr("SettingsView.settings")); //$NON-NLS-1$
        final GridLayout grid = new GridLayout(2, false);
        grid.verticalSpacing = 10;
        this.addTabs();
        // Die Buttons
        final ButtonArea buttons = new ButtonArea();
        buttons.addButton(I18N.tr("SettingsView.showAccount"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                final int selected = SettingsView.this.getTabFolder().getSelectionIndex();
                if (selected >= 0) {
                    final Konto account = SettingsView.this.tabs.get(selected).getAccount();
                    if (account == null) {
                        // show error message
                        final String msg = I18N.tr("SettingsController.noAccountSelected"); //$NON-NLS-1$
                        final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                        Application.getMessagingFactory().sendMessage(msgUser);
                    } else {
                        // Open account
                        GUI.startView(de.willuhn.jameica.hbci.gui.views.KontoNew.class, account);
                    }
                }
            }
        }, null, false, "document-open.png"); //$NON-NLS-1$
        buttons.addButton(I18N.tr("SettingsView.save"), new Action() { //$NON-NLS-1$

            /**
             * @see de.willuhn.jameica.gui.Action#handleAction(Object)
             */
            @Override
            public void handleAction(final Object context) throws ApplicationException {

                final int selected = SettingsView.this.getTabFolder().getSelectionIndex();
                if (selected == 0) {
                    // general settings --> call save methode
                    SettingsView.this.tabs.get(selected).getController().speichern();
                } else {
                    // first check if an account is selcted
                    final Konto account = SettingsView.this.tabs.get(selected).getAccount();
                    if (account == null) {
                        // show error message
                        final String msg = I18N.tr("SettingsController.noAccountSelected"); //$NON-NLS-1$
                        final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                        Application.getMessagingFactory().sendMessage(msgUser);
                    } else {
                        // call save methode
                        SettingsView.this.tabs.get(selected).getController().speichern();
                    }
                }
            }
        }, null, true, "document-save.png"); //$NON-NLS-1$
        buttons.paint(this.getParent());
    }

    /**
     * Aktualisiert die Liste mit allen Tabs
     */
    private void addTabs() {

        final Control[] oldTabs = this.getTabFolder().getChildren();
        for (final Control tab : oldTabs) {
            tab.dispose();
        }
        this.tabs.clear();
        this.tabs.add(new GeneralSettingsTabGroup(this.getTabFolder()));
        this.tabs.add(new BshSettingsTabGroup(this.getTabFolder()));
        this.tabs.add(new SantanderSettingsTabGroup(this.getTabFolder()));
        this.tabs.add(new TargoBankSettingsTabGroup(this.getTabFolder()));
        this.tabs.add(new BosSettingsTabGroup(this.getTabFolder()));
        this.tabs.add(new AktienSettingsTabGroup(this.getTabFolder()));
    }

    /**
     * Liefert den Tab-Folder, in dem die einzelnen Bankeinstellungen
     * untergebracht sind.
     *
     * @return der Tab-Folder.
     */
    private TabFolder getTabFolder() {

        if (this.folder == null) {
            this.folder = new TabFolder(this.getParent(), SWT.NONE);
            this.folder.setLayoutData(new GridData(GridData.FILL_BOTH));
        }
        return this.folder;
    }
}
