/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.pdfextractor;

import java.awt.Rectangle;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.apache.pdfbox.text.PDFTextStripperByArea;
import org.apache.pdfbox.text.TextPosition;

/**
 * Extended PDFTextStripperByArea class to add positions of each word to the
 * result string.
 *
 * @author Frank Mnster
 */
public abstract class AbstractBankPDFPage extends PDFTextStripperByArea {

    /**
     * Conversion value from CM to PDFBox values
     */
    public static final double CM_TO_PDF = 28.34645669;

    /**
     * Area-Name fr das Kartenkonto
     */
    public static final String KARTEN_KONTO = "kartenkonto"; //$NON-NLS-1$

    /**
     * Area-Name fr den Karteninhaber
     */
    public static final String KARTEN_INHABER = "karteninhaber"; //$NON-NLS-1$

    /**
     * Area-Name fr Rechungsdatum
     */
    public static final String RECHNUNGS_DATUM = "rechnungsdatum"; //$NON-NLS-1$

    /**
     * Area-Name fr Saldo
     */
    public static final String SALDO = "saldo"; //$NON-NLS-1$

    /**
     * Area-Name fr Buchungsdatum
     */
    public static final String BUCHUNGS_DATUM = "buchungsdatum"; //$NON-NLS-1$

    /**
     * Area-Name fr das Kaufdatum
     */
    public static final String KAUF_DATUM = "kaufdatum"; //$NON-NLS-1$

    /**
     * Area-Name fr die Beschreibung
     */
    public static final String BESCHREIBUNG = "beschreibung"; //$NON-NLS-1$

    /**
     * Area-Name fr die Fremdwhrung
     */
    public static final String FREMDWAEHRUNG = "fremdwaehrung"; //$NON-NLS-1$

    /**
     * Area-Name fr den Fremdwhrungsbetrag
     */
    public static final String FREMDW_BETRAG = "fremdwaehrungsbetrag"; //$NON-NLS-1$

    /**
     * Area-Name fr denKurs
     */
    public static final String KURS = "kurs"; //$NON-NLS-1$

    /**
     * Area-Name fr den Betrag
     */
    public static final String BETRAG = "betrag"; //$NON-NLS-1$

    /**
     * Area-Name fr den f&auml;lligen Betrag
     */
    public static final String FAELLIG_BETRAG = "faelligbetrag"; //$NON-NLS-1$

    /**
     * Area-Name fr das F&auml;lligkeitsdatum
     */
    public static final String FAELLIG_DATUM = "faelligdatum"; //$NON-NLS-1$

    /**
     * Area-Name fr die Empfnger IBAN der Ausgleichsbuchung
     */
    public static final String FAELLIG_IBAN = "faelligiban"; //$NON-NLS-1$

    /**
     * Area-Name fr die Empfnger BIC der Ausgleichsbuchung
     */
    public static final String FAELLIG_BIC = "faelligbic"; //$NON-NLS-1$

    /**
     * The delimiter in the return string from the PDFBox extraction separating
     * the positions and description values. Must be the same then the RegEx in
     * {@link AbstractPdfExtractor#WORT_DELIMITER}
     */
    private static final Object WORT_DELIMITER = "#*#"; //$NON-NLS-1$

    /**
     * Return the region as <code>Rectangle</code>. It converts centimetre into
     * PDF measure
     *
     * @param regionX
     * @param regionY
     * @param regionW
     * @param regionH
     * @return a <code>Rectangle</code> for defined region
     */
    protected static Rectangle getRectangleFrom(final double regionX, final double regionY, final double regionW,
            final double regionH) {

        final int recX = (int) (regionX * AbstractBankPDFPage.CM_TO_PDF);
        final int recY = (int) (regionY * AbstractBankPDFPage.CM_TO_PDF);
        final int recW = (int) (regionW * AbstractBankPDFPage.CM_TO_PDF);
        final int recH = (int) (regionH * AbstractBankPDFPage.CM_TO_PDF);
        return new Rectangle(recX, recY, recW, recH);
    }

    /**
     * Standard constructor calls super class.
     *
     * @throws IOException
     */
    protected AbstractBankPDFPage() throws IOException {
        super();
        this.setSortByPosition(true);
        this.setEndPage(1);
        this.setRegions();
    }

    /**
     * Defines the regions in a Map. The region rectangle values must able be
     * converted to PDF measures. Following key needs to be defined in the
     * {@code Map}:<br>
     * <ul>
     * <li>{@code AbstractBankPDFPage.BUCHUNGS_DATUM}</li>
     * <li>{@code AbstractBankPDFPage.KAUF_DATUM}</li>
     * <li>{@code AbstractBankPDFPage.BESCHREIBUNG}</li>
     * <li>{@code AbstractBankPDFPage.FREMDW_BETRAG}</li>
     * <li>{@code AbstractBankPDFPage.FREMDWAEHRUNG}</li>
     * <li>{@code AbstractBankPDFPage.KURS}</li>
     * <li>{@code AbstractBankPDFPage.BETRAG}</li>
     * </ul>
     *
     * @return a {@code Map} with the regions defined
     */
    protected abstract Map<String, Rectangle> defineRegions();

    /**
     * Liefert die zu erwartete X-Position des Buchungsdaten
     *
     * @return die zu erwartete X-Position des Buchungsdaten
     */
    protected abstract double getXPos();

    /**
     * Add the region to this TextStripper, but only if it is not
     * <code>NULL</code>
     *
     * @param regionName
     *            the name for the region
     * @param regions
     *            the rectangle defining the region
     */
    protected final void setRegion(final String regionName, final Map<String, Rectangle> regions) {

        if (regions.containsKey(regionName) && regions.get(regionName) != null) {
            if (this.getRegions().contains(regionName)) {
                this.removeRegion(regionName);
            }
            this.addRegion(regionName, regions.get(regionName));
        }

    }

    /**
     * Override the default functionality of PDFTextStripper.
     */
    @Override
    protected void writeString(final String text, final List<TextPosition> textPositions) throws IOException {

        final StringBuilder output = new StringBuilder();
        final String[] wordsInStream = text.split(this.getWordSeparator());
        if (wordsInStream != null) {
            for (final String word : wordsInStream) {
                final int textPos = text.indexOf(word);
                final TextPosition position = textPositions.get(textPos);
                output.append(' ').append(position.getTextMatrix().getTranslateX())
                        .append(AbstractBankPDFPage.WORT_DELIMITER).append(position.getTextMatrix().getTranslateY())
                        .append(AbstractBankPDFPage.WORT_DELIMITER).append(word);
            }
        }
        output.deleteCharAt(0);
        this.writeString(output.toString());
    }

    /**
     * Setzt die Regionen auf der Seite.
     */
    private void setRegions() {

        final Map<String, Rectangle> regions = this.defineRegions();

        this.setRegion(AbstractBankPDFPage.BUCHUNGS_DATUM, regions);
        this.setRegion(AbstractBankPDFPage.KAUF_DATUM, regions);
        this.setRegion(AbstractBankPDFPage.BESCHREIBUNG, regions);
        this.setRegion(AbstractBankPDFPage.FREMDW_BETRAG, regions);
        this.setRegion(AbstractBankPDFPage.FREMDWAEHRUNG, regions);
        this.setRegion(AbstractBankPDFPage.KURS, regions);
        this.setRegion(AbstractBankPDFPage.BETRAG, regions);

    }

}
