
package de.frankmuenster.mahoe.pdfextractor;

/**
 * Exception to report error during a PDF extraction
 *
 * @author Frank Mnster
 * @since 2.6.0
 *
 */
public class PdfExtractorException extends Exception {

    /**
     * Die Serial UID
     */
    private static final long serialVersionUID = 3682535546017597734L;

    /**
     * type of exception
     */
    private final transient PdfExtractorExceptionType type;

    /**
     * Creates a new exception with the given {@link PdfExtractorExceptionType}.
     *
     * @param type
     *            the type
     */
    public PdfExtractorException(final PdfExtractorExceptionType type) {
        this(null, type, null);
    }

    /**
     * Creates a new exception with the given {@link PdfExtractorExceptionType}
     * and cause.
     *
     * @param type
     *            the type
     * @param cause
     *            the cause. (A null value is permitted, and indicates that the
     *            cause is nonexistent or unknown.)
     */
    public PdfExtractorException(final PdfExtractorExceptionType type, final Throwable cause) {
        this(null, type, cause);
    }

    /**
     * Creates a new exception with the given message and
     * {@link PdfExtractorExceptionType}
     *
     * @param message
     *            the detail message
     * @param type
     *            the type
     */
    public PdfExtractorException(final String message, final PdfExtractorExceptionType type) {
        this(message, type, null);
    }

    /**
     * Creates a new exception with the given {@link PdfExtractorExceptionType}
     * and cause
     *
     * @param message
     *            the detail message
     * @param type
     *            the type
     * @param cause
     *            the cause. (A null value is permitted, and indicates that the
     *            cause is nonexistent or unknown.)
     */
    public PdfExtractorException(final String message, final PdfExtractorExceptionType type, final Throwable cause) {
        super(message, cause);
        this.type = type;
    }

    /**
     * Returns the {@link PdfExtractorExceptionType} of the exception.
     *
     *
     * @return the PdfExtractorExceptionType
     */
    public PdfExtractorExceptionType getType() {

        return this.type;
    }

}
