/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.utils;

import java.rmi.RemoteException;

import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.server.DBPropertyUtil;

/**
 * Utility class for extended BD functions
 *
 * @author Frank Mnster
 *
 */
public final class DBUtiles {

    /**
     * Removes a META key from an account
     *
     * @param account
     *            the account
     * @param metaKey
     *            the meta key to remove
     * @return the number of the removed keys (can be only 0 or 1)
     * @throws RemoteException
     */
    public static int deleteMeta(final Konto account, final String metaKey) throws RemoteException {

        if (account == null) {
            throw new RemoteException(I18N.tr("DBUtiles.noAccountGiven")); //$NON-NLS-1$
        }

        if (metaKey == null || metaKey.length() == 0) {
            throw new RemoteException(I18N.tr("DBUtiles.noMetakeyGiven")); //$NON-NLS-1$
        }

        if (metaKey.indexOf('%') != -1 || metaKey.indexOf('_') != -1) {
            throw new RemoteException(I18N.tr("DBUtiles.noWildcardsAllowed")); //$NON-NLS-1$
        }

        final StringBuilder localPrefix = new StringBuilder(DBPropertyUtil.Prefix.META.value());
        localPrefix.append(".konto.").append(account.getID()).append('.').append(metaKey); //$NON-NLS-1$

        return Settings.getDBService().executeUpdate("delete from property where name = ?", localPrefix.toString()); //$NON-NLS-1$

    }

    /**
     * Standard constructor
     */
    private DBUtiles() {
        // Utility class - intentionally left blank
    }

}
