
package de.frankmuenster.mahoe.web;

import java.util.Date;
import java.util.List;

import de.willuhn.jameica.hbci.rmi.Umsatz;

/**
 * Represents a web page with the account statements. It can be used to either
 * download the the statements or to extract the transactions from the
 * statements.
 *
 * @author Frank Mnster
 * @since 2.4.0
 *
 */
public interface BankAccountStatementPage {

    /**
     * Exists a NEXT button for the last getAccountLinks call.
     *
     * @return <b>true</b> in case a NEXT button exists with possible PDFs after
     *         the given start date, <b>false</b> else
     */
    boolean accountLinksHasNextPage();

    /**
     * Open the next page if exists
     *
     * @return same object with the next page loaded
     */
    BankAccountStatementPage getNextPage();

    /**
     * Read all the transactions from the PDF from the given start date
     *
     * @param startDate
     *            the date to start reading the transactions from
     * @return a List with all found transactions
     * @throws WebSiteException
     *             in case the transactions could not be read
     */
    List<Umsatz> getTransactions(Date startDate) throws WebSiteException;

    /**
     * Exists a NEXT button, so a next page
     *
     * @return <b>true</b> in case a NEXT button exists, <b>false</b> else
     */
    boolean hasNextPage();

}