/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.web;

import java.util.List;

import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.util.ApplicationException;

/**
 * Interface fr die Kommunikation mit der Bank-Webseite
 *
 * @author Frank Mnster
 * @param <I>
 *            die bankspezifischen WebAccountInformation
 */
public interface WebSite<I extends WebAccountInformation> {

    /**
     * Erstellt eine Sepa-berweisung bei der Bank.
     *
     * @param aUeberweisung
     *            die Sepa-berweisung.
     * @throws WebSiteException
     * @throws ApplicationException
     */
    void createSepaTransfer(final AuslandsUeberweisung aUeberweisung) throws WebSiteException, ApplicationException;

    /**
     * Liefert kontospezifische Informationen, die auf alle Flle das Saldo und
     * das letzte Umsatzdatum enthalten.
     *
     * @return die kontospezifische Informationen
     * @throws WebSiteException
     * @throws ApplicationException
     */
    I getAccountInformation() throws WebSiteException, ApplicationException;

    /**
     * Returns a list of new accounts sheets to save.
     *
     * @param accountInfo
     *            the account information
     * @return a list with new account sheets
     * @throws WebSiteException
     */
    List<MahoeKontoauszugPDF> getAccountSheets(final I accountInfo) throws WebSiteException;

    /**
     * Liefert eine Liste mit den gebuchten Umstzen.
     *
     * @return Liste mit den gefunden Umstzen, eine leere Liste sonst.
     * @throws WebSiteException
     * @throws ApplicationException
     */
    List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException;

    /**
     * Ldt neue Bank-Nachrichten
     *
     * @param accountInfo
     * @return die Anzahl an gespeicherten Nachrichten
     * @throws WebSiteException
     * @throws ApplicationException
     */
    int getMessages(final I accountInfo) throws WebSiteException, ApplicationException;

    /**
     * Liefert eine Liste mit den vorgemerkten Umstzen.
     *
     * @return Liste mit den gefunden Umstzen, eine leere Liste sonst.
     * @throws WebSiteException
     * @throws ApplicationException
     */
    List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException;

    /**
     * berprft, ob das Webseite-Object immer noch an der Webseite angemeldet
     * ist.
     *
     * @return true wenn die Session noch aktiv ist, false sonst
     * @throws WebSiteException
     *
     * @deprecated Use the Login Page instead
     */
    @Deprecated
    boolean isStillLoggedIn() throws WebSiteException;

    /**
     * An der Bank-Seite anmelden
     *
     * @param password
     *            das Passwort
     * @param loginParams
     *            Login-Parameter als Array, das verschiedene Banken
     *            verschiedene Anforderungen haben knnen
     * @throws WebSiteException
     */
    void login(final char[] password, String... loginParams) throws WebSiteException;

    /**
     * Abmelden von der Bank-Seite
     *
     * @throws WebSiteException
     */
    void logoff() throws WebSiteException;

}
