/**
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.web.webpages;

import org.openqa.selenium.WebDriver;

import de.frankmuenster.mahoe.web.WebSiteException;

/**
 * A Selenium account page
 *
 * @author Frank Muenster
 *
 * @param <I>
 */
public abstract class AbstractAccountPage<I> extends SeleniumWebPage {

    /**
     * The account information to open from and update with this Selenium page
     */
    protected final I accountInfo;

    /**
     * Creates a new Selenium account web page for the given account
     *
     * @param driver
     *            the Selenium driver
     * @param accountInfo
     *            the account for this page
     */
    protected AbstractAccountPage(final WebDriver driver, final I accountInfo) {
        super(driver);

        this.accountInfo = accountInfo;

    }

    /**
     * Open the account page for the given account
     *
     * @throws WebSiteException
     *             in case the page cannot be opened
     */
    protected abstract void open() throws WebSiteException;

}
