/**
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.web.webpages;

import java.math.BigDecimal;
import java.time.LocalDate;

import de.frankmuenster.mahoe.web.WebSiteException;

/**
 * Interface to get the account details from the bank web site
 * 
 * @author Frank Mnster
 *
 * @since 2.6.0
 *
 */
public interface DetailsPage {

    /**
     * Returns the available amount.
     *
     * @return the available amount
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    BigDecimal getAvailableAmount() throws WebSiteException;

    /**
     * Return the limit of the credit card.
     *
     * @return the credit card limit
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    BigDecimal getCreditLimit() throws WebSiteException;

    /**
     * Return the date of the last transaction.
     *
     * @return date of the last transaction
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    LocalDate getLastTransaction() throws WebSiteException;

    /**
     * Return the date of the next credit card bill.
     *
     * @return date of the next credit card bill
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    LocalDate getNextBill() throws WebSiteException;

    /**
     * Return the date of the next automatic subtraction.
     *
     * @return date of the next automatic subtraction
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    LocalDate getNextSubtraction() throws WebSiteException;

    /**
     * The description of the credit card.
     *
     * @return the credit card description
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    String getProductDescription() throws WebSiteException;

    /**
     * The actual saldo.
     *
     * @return the saldo
     * @throws WebSiteException
     *             in case the value could not be read from the details hover
     *             page
     */
    BigDecimal getSaldo() throws WebSiteException;

}